@@ -20,21 +20,24 @@ import mock
 import os
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
 
-from cinderclient.v2 import client as cinderclient
+from cinderclient.v3 import client as cinderclient
 from os_brick.initiator import connector
 from oslo_concurrency import processutils
 from oslo_utils import units
 
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder  # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
@@ -61,9 +64,10 @@ class TestCinderStore(base.StoreBaseTest,
                                   user='fake_user',
                                   auth_token='fake_token',
                                   tenant='fake_tenant')
+        self.hash_algo = 'sha256'
 
     def test_get_cinderclient(self):
-        cc = cinder.get_cinderclient(self.conf, self.context)
+        cc = self.store.get_cinderclient(self.context)
         self.assertEqual('fake_token', cc.client.auth_token)
         self.assertEqual('http://foo/public_url', cc.client.management_url)
 
@@ -72,7 +76,7 @@ class TestCinderStore(base.StoreBaseTest,
         self.config(cinder_store_password='test_password')
         self.config(cinder_store_project_name='test_project')
         self.config(cinder_store_auth_address='test_address')
-        cc = cinder.get_cinderclient(self.conf, self.context)
+        cc = self.store.get_cinderclient(self.context)
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
@@ -94,9 +98,9 @@ class TestCinderStore(base.StoreBaseTest,
         with mock.patch.object(os, 'stat', return_value=fake_stat()), \
                 mock.patch.object(os, 'getuid', return_value=2), \
                 mock.patch.object(processutils, 'execute') as mock_execute, \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value='sudo'):
-            with cinder.temporary_chown('test'):
+            with self.store.temporary_chown('test'):
                 pass
             expected_calls = [mock.call('chown', 2, 'test', run_as_root=True,
                                         root_helper='sudo'),
@@ -144,7 +148,10 @@ class TestCinderStore(base.StoreBaseTest,
                           volume_available, 'available', 'in-use')
         fake_manager.get.assert_called_with('fake-id')
 
-    def _test_open_cinder_volume(self, open_mode, attach_mode, error):
+    def _test_open_cinder_volume(self, open_mode, attach_mode, error,
+                                 multipath_supported=False,
+                                 enforce_multipath=False):
+        self.config(cinder_mount_point_base=None)
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
         fake_volumes = FakeObject(get=lambda id: fake_volume,
                                   detach=mock.Mock())
@@ -174,26 +181,30 @@ class TestCinderStore(base.StoreBaseTest,
         with mock.patch.object(cinder.Store,
                                '_wait_volume_status',
                                return_value=fake_volume), \
-                mock.patch.object(cinder, 'temporary_chown',
+                mock.patch.object(cinder.Store, 'temporary_chown',
                                   side_effect=fake_chown), \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
-                mock.patch.object(connector, 'get_connector_properties'), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
                                   side_effect=fake_factory):
 
-            if error:
-                self.assertRaises(error, do_open)
-            else:
-                do_open()
+            with mock.patch.object(connector,
+                                   'get_connector_properties') as mock_conn:
+                if error:
+                    self.assertRaises(error, do_open)
+                else:
+                    do_open()
 
-            fake_connector.connect_volume.assert_called_once_with(mock.ANY)
-            fake_connector.disconnect_volume.assert_called_once_with(
-                mock.ANY, fake_devinfo)
-            fake_volume.attach.assert_called_once_with(
-                None, 'glance_store', attach_mode,
-                host_name=socket.gethostname())
-            fake_volumes.detach.assert_called_once_with(fake_volume)
+                mock_conn.assert_called_once_with(
+                    root_helper, socket.gethostname(), multipath_supported,
+                    enforce_multipath)
+                fake_connector.connect_volume.assert_called_once_with(mock.ANY)
+                fake_connector.disconnect_volume.assert_called_once_with(
+                    mock.ANY, fake_devinfo)
+                fake_volume.attach.assert_called_once_with(
+                    None, 'glance_store', attach_mode,
+                    host_name=socket.gethostname())
+                fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -204,6 +215,18 @@ class TestCinderStore(base.StoreBaseTest,
     def test_open_cinder_volume_error(self):
         self._test_open_cinder_volume('wb', 'rw', IOError)
 
+    def test_open_cinder_volume_multipath_supported(self):
+        self.config(cinder_use_multipath=True)
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True)
+
+    def test_open_cinder_volume_enforce_multipath(self):
+        self.config(cinder_use_multipath=True)
+        self.config(cinder_enforce_multipath=True)
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True,
+                                      enforce_multipath=True)
+
     def test_cinder_configure_add(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
@@ -231,7 +254,7 @@ class TestCinderStore(base.StoreBaseTest,
             self.assertEqual('rb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
@@ -257,7 +280,7 @@ class TestCinderStore(base.StoreBaseTest,
         fake_volume = FakeObject(size=5, metadata={})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -274,7 +297,7 @@ class TestCinderStore(base.StoreBaseTest,
                                  metadata={'image_size': expected_image_size})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -290,6 +313,7 @@ class TestCinderStore(base.StoreBaseTest,
         expected_file_contents = b"*" * expected_size
         image_file = six.BytesIO(expected_file_contents)
         expected_checksum = hashlib.md5(expected_file_contents).hexdigest()
+        expected_multihash = hashlib.sha256(expected_file_contents).hexdigest()
         expected_location = 'cinder://%s' % fake_volume.id
         fake_client = FakeObject(auth_token=None, management_url=None)
         fake_volume.manager.get.return_value = fake_volume
@@ -301,19 +325,18 @@ class TestCinderStore(base.StoreBaseTest,
             self.assertEqual('wb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
                                               volumes=fake_volumes)
-            loc, size, checksum, _ = self.store.add(expected_image_id,
-                                                    image_file,
-                                                    expected_size,
-                                                    self.context,
-                                                    verifier)
+            loc, size, checksum, multihash, _ = self.store.add(
+                expected_image_id, image_file, expected_size, self.hash_algo,
+                self.context, verifier)
             self.assertEqual(expected_location, loc)
             self.assertEqual(expected_size, size)
             self.assertEqual(expected_checksum, checksum)
+            self.assertEqual(expected_multihash, multihash)
             fake_volumes.create.assert_called_once_with(
                 1,
                 name='image-%s' % expected_image_id,
@@ -356,7 +379,7 @@ class TestCinderStore(base.StoreBaseTest,
         fake_volume = FakeObject(delete=mock.Mock())
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
