@@ -20,6 +20,7 @@ import mock
 import os
 import six
 import socket
+import sys
 import tempfile
 import time
 import uuid
@@ -31,12 +32,14 @@ from oslo_config import cfg
 from oslo_utils import units
 
 import glance_store as store
-from glance_store._drivers import cinder
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
 from glance_store.tests.unit import test_store_capabilities as test_cap
 
+sys.modules['glance_store.common.fs_mount'] = mock.Mock()
+from glance_store._drivers import cinder  # noqa
+
 
 class FakeObject(object):
     def __init__(self, **kwargs):
@@ -93,8 +96,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         self.assertEqual("cinder://", self.store.url_prefix)
 
     def test_get_cinderclient(self):
-        cc = cinder.get_cinderclient(self.conf, self.context,
-                                     backend="cinder1")
+        cc = self.store.get_cinderclient(self.context)
         self.assertEqual('fake_token', cc.client.auth_token)
         self.assertEqual('http://foo/public_url', cc.client.management_url)
 
@@ -103,8 +105,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         self.config(cinder_store_password='test_password', group="cinder1")
         self.config(cinder_store_project_name='test_project', group="cinder1")
         self.config(cinder_store_auth_address='test_address', group="cinder1")
-        cc = cinder.get_cinderclient(self.conf, self.context,
-                                     backend="cinder1")
+        cc = self.store.get_cinderclient(self.context)
         self.assertIsNone(cc.client.auth_token)
         self.assertEqual('test_address', cc.client.management_url)
 
@@ -115,9 +116,9 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         with mock.patch.object(os, 'stat', return_value=fake_stat()), \
                 mock.patch.object(os, 'getuid', return_value=2), \
                 mock.patch.object(processutils, 'execute') as mock_execute, \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value='sudo'):
-            with cinder.temporary_chown('test'):
+            with self.store.temporary_chown('test'):
                 pass
             expected_calls = [mock.call('chown', 2, 'test', run_as_root=True,
                                         root_helper='sudo'),
@@ -165,7 +166,10 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                           volume_available, 'available', 'in-use')
         fake_manager.get.assert_called_with('fake-id')
 
-    def _test_open_cinder_volume(self, open_mode, attach_mode, error):
+    def _test_open_cinder_volume(self, open_mode, attach_mode, error,
+                                 multipath_supported=False,
+                                 enforce_multipath=False):
+        self.config(cinder_mount_point_base=None, group='cinder1')
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()), status='available')
         fake_volumes = FakeObject(get=lambda id: fake_volume,
                                   detach=mock.Mock())
@@ -195,26 +199,30 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         with mock.patch.object(cinder.Store,
                                '_wait_volume_status',
                                return_value=fake_volume), \
-                mock.patch.object(cinder, 'temporary_chown',
+                mock.patch.object(cinder.Store, 'temporary_chown',
                                   side_effect=fake_chown), \
-                mock.patch.object(cinder, 'get_root_helper',
+                mock.patch.object(cinder.Store, 'get_root_helper',
                                   return_value=root_helper), \
-                mock.patch.object(connector, 'get_connector_properties'), \
                 mock.patch.object(connector.InitiatorConnector, 'factory',
                                   side_effect=fake_factory):
 
-            if error:
-                self.assertRaises(error, do_open)
-            else:
-                do_open()
+            with mock.patch.object(connector,
+                                   'get_connector_properties') as mock_conn:
+                if error:
+                    self.assertRaises(error, do_open)
+                else:
+                    do_open()
 
-            fake_connector.connect_volume.assert_called_once_with(mock.ANY)
-            fake_connector.disconnect_volume.assert_called_once_with(
-                mock.ANY, fake_devinfo)
-            fake_volume.attach.assert_called_once_with(
-                None, 'glance_store', attach_mode,
-                host_name=socket.gethostname())
-            fake_volumes.detach.assert_called_once_with(fake_volume)
+                mock_conn.assert_called_once_with(
+                    root_helper, socket.gethostname(), multipath_supported,
+                    enforce_multipath)
+                fake_connector.connect_volume.assert_called_once_with(mock.ANY)
+                fake_connector.disconnect_volume.assert_called_once_with(
+                    mock.ANY, fake_devinfo)
+                fake_volume.attach.assert_called_once_with(
+                    None, 'glance_store', attach_mode,
+                    host_name=socket.gethostname())
+                fake_volumes.detach.assert_called_once_with(fake_volume)
 
     def test_open_cinder_volume_rw(self):
         self._test_open_cinder_volume('wb', 'rw', None)
@@ -225,6 +233,19 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
     def test_open_cinder_volume_error(self):
         self._test_open_cinder_volume('wb', 'rw', IOError)
 
+    def test_open_cinder_volume_multipath_disabled(self):
+        self.config(cinder_use_multipath=False, group='cinder1')
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=False)
+
+    def test_open_cinder_volume_enforce_multipath(self):
+
+        self.config(cinder_use_multipath=True, group='cinder1')
+        self.config(cinder_enforce_multipath=True, group='cinder1')
+        self._test_open_cinder_volume('wb', 'rw', None,
+                                      multipath_supported=True,
+                                      enforce_multipath=True)
+
     def test_cinder_configure_add(self):
         self.assertRaises(exceptions.BadStoreConfiguration,
                           self.store._check_context, None)
@@ -252,7 +273,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual('rb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
@@ -280,7 +301,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_volume = FakeObject(size=5, metadata={})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -299,7 +320,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                                  metadata={'image_size': expected_image_size})
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
@@ -328,7 +349,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
             self.assertEqual('wb', mode)
             yield volume_file
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mock_cc, \
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mock_cc, \
                 mock.patch.object(self.store, '_open_cinder_volume',
                                   side_effect=fake_open):
             mock_cc.return_value = FakeObject(client=fake_client,
@@ -348,7 +369,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
                           'glance_image_id': expected_image_id,
                           'image_size': str(expected_size)},
                 volume_type='some_type')
-            self.assertEqual(backend, metadata["backend"])
+            self.assertEqual(backend, metadata["store"])
 
     def test_cinder_add(self):
         fake_volume = mock.MagicMock(id=str(uuid.uuid4()),
@@ -384,7 +405,7 @@ class TestMultiCinderStore(base.MultiStoreBaseTest,
         fake_volume = FakeObject(delete=mock.Mock())
         fake_volumes = {fake_volume_uuid: fake_volume}
 
-        with mock.patch.object(cinder, 'get_cinderclient') as mocked_cc:
+        with mock.patch.object(cinder.Store, 'get_cinderclient') as mocked_cc:
             mocked_cc.return_value = FakeObject(client=fake_client,
                                                 volumes=fake_volumes)
 
