@@ -35,6 +35,9 @@ class MockRados(object):
     class Error(Exception):
         pass
 
+    class ObjectNotFound(Exception):
+        pass
+
     class ioctx(object):
         def __init__(self, *args, **kwargs):
             pass
@@ -228,7 +231,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 self.assertTrue(resize.called)
                 self.assertTrue(write.called)
                 self.assertEqual(ret[1], self.data_len)
-                self.assertEqual("ceph1", ret[3]['backend'])
+                self.assertEqual("ceph1", ret[3]['store'])
 
     def test_add_w_image_size_zero_to_different_backend(self):
         """Assert that correct size is returned even though 0 was provided."""
@@ -252,7 +255,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 self.assertTrue(resize.called)
                 self.assertTrue(write.called)
                 self.assertEqual(ret[1], self.data_len)
-                self.assertEqual("ceph2", ret[3]['backend'])
+                self.assertEqual("ceph2", ret[3]['store'])
 
     @mock.patch.object(MockRBD.Image, '__enter__')
     @mock.patch.object(rbd_store.Store, '_create_image')
@@ -442,11 +445,12 @@ class TestMultiStore(base.MultiStoreBaseTest,
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
+        rbd_store.rados.ObjectNotFound = MockRados.ObjectNotFound
 
         def test():
             with self.store.get_connection('conffile', 'rados_id'):
                 pass
-        self.assertRaises(exceptions.BackendException, test)
+        self.assertRaises(exceptions.BadStoreConfiguration, test)
 
     def test_create_image_conf_features(self):
         # Tests that we use non-0 features from ceph.conf and cast to int.
